@@ -3,7 +3,6 @@
 
 use getopts::Options;
 use std::env;
-use std::error::Error as StdError;
 use std::fs;
 use std::io::{self, BufRead, BufReader, BufWriter, Read, Write};
 use std::ops::AddAssign;
@@ -19,6 +18,7 @@ use std::time::{Duration, SystemTime};
 
 use anyhow::{anyhow, Context};
 
+use nix::sys::signal::{sigprocmask, SigSet, SigmaskHow::*, Signal::*};
 use nix::unistd::{chown, execve, Uid, User};
 #[cfg(target_os = "linux")]
 use nix::unistd::{setresgid, setresuid};
@@ -88,13 +88,13 @@ struct Logger {
 }
 
 impl Logger {
-    fn log<S: Serialize>(&mut self, message: S) {
+    fn log<S: Serialize>(&mut self, message: S) -> std::io::Result<()> {
         if let Some(prefix) = &self.prefix {
-            self.output.write_all(prefix.as_bytes()).unwrap();
+            self.output.write_all(prefix.as_bytes())?;
         }
-        serde_json::to_writer(&mut self.output, &message).unwrap();
-        self.output.write_all(b"\n").unwrap();
-        self.output.flush().unwrap();
+        laurel::json::to_writer(&mut self.output, &message)?;
+        self.output.write_all(b"\n")?;
+        self.output.flush()
     }
 
     fn new(def: &Logfile, dir: &Path) -> anyhow::Result<Self> {
@@ -134,7 +134,7 @@ impl Logger {
     }
 }
 
-fn run_app() -> Result<(), Box<dyn StdError>> {
+fn run_app() -> Result<(), anyhow::Error> {
     let args: Vec<String> = env::args().collect();
 
     let mut opts = Options::new();
@@ -155,19 +155,19 @@ fn run_app() -> Result<(), Box<dyn StdError>> {
     }
 
     let config: Config = match matches.opt_str("c") {
-        Some(f_name) => {
-            if fs::metadata(&f_name)
-                .with_context(|| format!("stat: {f_name}"))?
+        Some(f) => {
+            if fs::metadata(&f)
+                .with_context(|| format!("stat {f}"))?
                 .permissions()
                 .mode()
                 & 0o002
                 != 0
             {
-                return Err(format!("Config file {} must not be world-writable", f_name).into());
+                return Err(anyhow!("Config file {f} must not be world-writable"));
             }
-            let lines = fs::read(&f_name).with_context(|| format!("read(: {f_name}"))?;
-            toml::from_str(&String::from_utf8(lines).with_context(|| format!("parse: {f_name}"))?)
-                .with_context(|| format!("parse {f_name}"))?
+            let lines = fs::read(&f).with_context(|| format!("Error reading {f}"))?;
+            toml::from_str(&String::from_utf8(lines).with_context(|| format!("Error parsing {f}"))?)
+                .with_context(|| format!("Error parsing {f}"))?
         }
         None => Config::default(),
     };
@@ -180,7 +180,7 @@ fn run_app() -> Result<(), Box<dyn StdError>> {
         Input::Stdin => Box::new(unsafe { std::fs::File::from_raw_fd(0) }),
         Input::Unix(path) => Box::new(
             UnixStream::connect(path)
-                .with_context(|| format!("connect: {}", path.to_string_lossy()))?,
+                .with_context(|| format!("Error connecting to {}", path.to_string_lossy()))?,
         ),
     };
 
@@ -190,11 +190,11 @@ fn run_app() -> Result<(), Box<dyn StdError>> {
 
     let runas_user = match config.user {
         Some(ref username) => {
-            User::from_name(username)?.ok_or_else(|| format!("user {} not found", username))?
+            User::from_name(username)?.ok_or_else(|| anyhow!("user {username} not found"))?
         }
         None => {
             let uid = Uid::effective();
-            User::from_uid(uid)?.ok_or_else(|| format!("uid {} not found", uid))?
+            User::from_uid(uid)?.ok_or_else(|| anyhow!("uid {uid} not found"))?
         }
     };
 
@@ -209,7 +209,7 @@ fn run_app() -> Result<(), Box<dyn StdError>> {
         .unwrap_or_else(|| Path::new(".").to_path_buf());
     if dir.exists() {
         if !dir.is_dir() {
-            return Err(format!("{} is not a directory", dir.to_string_lossy()).into());
+            return Err(anyhow!("{} is not a directory", dir.to_string_lossy()));
         }
         if dir
             .metadata()
@@ -220,7 +220,7 @@ fn run_app() -> Result<(), Box<dyn StdError>> {
             != 0
         {
             log::warn!(
-                "Base directory {} must not be world-wirtable",
+                "Base directory {} must not be world-writable",
                 dir.to_string_lossy()
             );
         }
@@ -245,7 +245,7 @@ fn run_app() -> Result<(), Box<dyn StdError>> {
         for user in &def.clone().users.unwrap_or_default() {
             rot = rot.with_uid(
                 User::from_name(user)?
-                    .ok_or_else(|| format!("user {} not found", &user))?
+                    .ok_or_else(|| anyhow!("user {user} not found"))?
                     .uid,
             );
         }
@@ -270,7 +270,7 @@ fn run_app() -> Result<(), Box<dyn StdError>> {
     }
     #[cfg(target_os = "linux")]
     if let Err(e) = caps::clear(None, CapSet::Ambient) {
-        log::warn!("could not set ambient capabilities: {}", e);
+        log::warn!("could not set ambient capabilities: {e}");
     }
 
     // Initial setup is done at this point.
@@ -297,9 +297,9 @@ fn run_app() -> Result<(), Box<dyn StdError>> {
             Logger::new(&config.filterlog, &dir).context("can't create filterlog logger")?;
         emit_fn_log = move |e: &Event| {
             if e.filter {
-                filter_logger.log(e)
+                filter_logger.log(e).expect("Error writing to filter log");
             } else {
-                logger.log(e)
+                logger.log(e).expect("Error writing to audit log");
             }
         };
         coalesce = Coalesce::new(emit_fn_log);
@@ -307,14 +307,14 @@ fn run_app() -> Result<(), Box<dyn StdError>> {
         log::info!("Dropping filtered audit records");
         emit_fn_drop = move |e: &Event| {
             if !e.filter {
-                logger.log(e)
+                logger.log(e).expect("Error writing to audit log");
             }
         };
         coalesce = Coalesce::new(emit_fn_drop);
     }
 
     coalesce.settings = config.make_coalesce_settings();
-    coalesce.initialize()?;
+    coalesce.initialize().context("Failed to initialize")?;
 
     let mut line: Vec<u8> = Vec::new();
     let mut stats = Stats::default();
@@ -326,6 +326,7 @@ fn run_app() -> Result<(), Box<dyn StdError>> {
     let dump_state_period = config.debug.dump_state_period.map(Duration::from_secs);
     let mut dump_state_last_t = SystemTime::now();
 
+    sigprocmask(SIG_UNBLOCK, Some(&SigSet::from_iter([SIGHUP])), None)?;
     let hup = Arc::new(AtomicBool::new(false));
     signal_hook::flag::register(signal_hook::consts::SIGHUP, Arc::clone(&hup))?;
 
@@ -335,14 +336,14 @@ fn run_app() -> Result<(), Box<dyn StdError>> {
             let lines = buf.split_inclusive(|c| *c == b'\n');
             log::info!("Got SIGHUP.");
             for line in lines {
-                if let Err(e) = coalesce.process_line(line.to_vec()) {
+                if let Err(e) = coalesce.process_line(line) {
                     if let Some(ref mut l) = error_logger {
                         l.write_all(line)
                             .and_then(|_| l.flush())
                             .context("write log")?;
                     }
                     let line = String::from_utf8_lossy(line).replace('\n', "");
-                    log::error!("Error {} processing msg: {}", e, &line);
+                    log::error!("Error {e} processing msg: {line}");
                 }
             }
             coalesce.flush();
@@ -350,7 +351,7 @@ fn run_app() -> Result<(), Box<dyn StdError>> {
             use std::ffi::CString;
             let argv: Vec<CString> = env::args().map(|a| CString::new(a).unwrap()).collect();
             let env: Vec<CString> = env::vars()
-                .map(|(k, v)| CString::new(format!("{}={}", k, v)).unwrap())
+                .map(|(k, v)| CString::new(format!("{k}={v}")).unwrap())
                 .collect();
 
             #[cfg(target_os = "linux")]
@@ -359,7 +360,7 @@ fn run_app() -> Result<(), Box<dyn StdError>> {
                 capabilities.insert(Capability::CAP_SYS_PTRACE);
                 capabilities.insert(Capability::CAP_DAC_READ_SEARCH);
                 if let Err(e) = caps::set(None, CapSet::Ambient, &capabilities) {
-                    log::warn!("could not set ambient capabilities: {}", e);
+                    log::warn!("could not set ambient capabilities: {e}");
                 }
             }
             execve(&argv[0], &argv, &env)?;
@@ -375,7 +376,7 @@ fn run_app() -> Result<(), Box<dyn StdError>> {
         }
 
         stats.lines += 1;
-        match coalesce.process_line(line.clone()) {
+        match coalesce.process_line(&line) {
             Ok(()) => (),
             Err(e) => {
                 stats.errors += 1;
@@ -385,7 +386,7 @@ fn run_app() -> Result<(), Box<dyn StdError>> {
                         .context("write log")?;
                 }
                 let line = String::from_utf8_lossy(&line).replace('\n', "");
-                log::error!("Error {} processing msg: {}", e, &line);
+                log::error!("Error {e} processing msg: {line}");
                 continue;
             }
         };
@@ -414,8 +415,7 @@ fn run_app() -> Result<(), Box<dyn StdError>> {
             if dump_state_last_t.elapsed()? >= *p {
                 coalesce
                     .dump_state(&mut dl.output)
-                    // .context("dump state")?;
-                    .map_err(|e| format!("dump state: {}", e))?;
+                    .map_err(|e| anyhow!("dump state: {e}"))?;
                 dump_state_last_t = SystemTime::now();
             }
         }
@@ -461,17 +461,15 @@ pub fn main() {
                 Some(l) => format!("{}:{},{}", l.file(), l.line(), l.column()),
                 None => "(unknown)".to_string(),
             };
-            let e = format!("fatal error '{}' at {}", &message, &location);
-            log::error!("{}", &e);
+            log::error!("fatal error '{message}' at {location}");
         }));
     }
 
     match run_app() {
         Ok(_) => (),
         Err(e) => {
-            let e = e.to_string();
-            log::error!("{}", &e);
-            std::process::abort();
+            log::error!("{e:#}");
+            std::process::exit(1);
         }
     };
 }
