@@ -62,7 +62,7 @@ impl FileRotate {
             let mut old = self.basename.clone();
             match suffix {
                 0 => (),
-                _ => old.push(format!(".{}", suffix)),
+                _ => old.push(format!(".{suffix}")),
             };
             let mut new = self.basename.clone();
             new.push(format!(".{}", suffix + 1));
@@ -87,14 +87,10 @@ impl FileRotate {
             AclEntry::allow_other(Perm::empty(), None),
         ];
         for uid in &self.uids {
-            acl.push(AclEntry::allow_user(&format!("{}", uid), Perm::READ, None));
+            acl.push(AclEntry::allow_user(&format!("{uid}"), Perm::READ, None));
         }
 
-        if let Ok(mut f) = OpenOptions::new()
-            .write(true)
-            .append(true)
-            .open(&self.basename)
-        {
+        if let Ok(mut f) = OpenOptions::new().append(true).open(&self.basename) {
             fchmod(f.as_raw_fd(), Mode::from_bits(0o600).unwrap())
                 .map_err(|e| Error::new(ErrorKind::Other, e))?;
 
@@ -114,7 +110,6 @@ impl FileRotate {
             let f = OpenOptions::new()
                 .create_new(true)
                 .mode(0o600)
-                .write(true)
                 .append(true)
                 .open(&tmp)?;
 
